@@ -1,5 +1,5 @@
 import logging
-from flask import Blueprint
+from flask import Blueprint, request, redirect, session, url_for
 from flask.globals import _app_ctx_stack, _request_ctx_stack
 from werkzeug.urls import url_parse
 from .forms import GeneralModelConverter
@@ -140,40 +140,27 @@ class BaseView(object):
         return name.replace('_', ' ').title()
 
 
-    def route_from(url, method=None):
-        appctx = _app_ctx_stack.top
-        reqctx = _request_ctx_stack.top
-        if appctx is None:
-            raise RuntimeError('Attempted to match a URL without the '
-                               'application context being pushed. This has to be '
-                               'executed when application context is available.')
+    def update_redirect(self):
+        """
+            Call it on your own endpoint's to update the back history navigation.
+            If you bypass it, the next submit our back will go over it.
+        """
+        page_history = Stack(session.get('page_history', []))
+        page_history.push(request.url)
+        session['page_history'] = page_history.to_json()
+        
+    def get_redirect(self):
+        """
+            Returns the previous url.
+        """
+        index_url = url_for('%s.%s' % (self.appbuilder.indexview.endpoint, self.appbuilder.indexview.default_view))
+        page_history = Stack(session.get('page_history', []))
 
-        if reqctx is not None:
-            url_adapter = reqctx.url_adapter
-        else:
-            url_adapter = appctx.url_adapter
-            if url_adapter is None:
-                raise RuntimeError('Application was not able to create a URL '
-                                   'adapter for request independent URL matching. '
-                                   'You might be able to fix this by setting '
-                                   'the SERVER_NAME config variable.')
-        parsed_url = url_parse(url)
-        if parsed_url.netloc is not "" and parsed_url.netloc != url_adapter.server_name:
-            raise NotFound()
-        return url_adapter.match(parsed_url.path, method)
-
-    def _get_redirect(self):
-        next_url = request.args.get('next')
-        if next_url:
-            if next_url in request.referrer:
-                return request.referrer
-            else:
-                return request.args.get('next')
-        else:
-            try:
-                return url_for('%s.%s' % (self.endpoint, self.default_view), **request.args)
-            except:
-                return url_for('%s.%s' % (self.appbuilder.indexview.endpoint, self.appbuilder.indexview.default_view))
+        if page_history.pop() is None:
+            return index_url
+        session['page_history'] = page_history.to_json()
+        redir = page_history.pop() or index_url
+        return redir
 
 
 class BaseModelView(BaseView):
@@ -380,6 +367,28 @@ class BaseCRUDView(BaseModelView):
     """ Dictionary to add extra fields to the Add form using this property """
     edit_form_extra_fields = None
     """ Dictionary to Add extra fields to the Edit form using this property """
+    add_form_query_cascade = None
+    """
+        FUTURE FEATURE, Don't use it yet
+        
+        Implements query cascade related fields. Will user relate fields
+        with multiple values.
+
+        Grammar: [('Parent field','Child field', SQLAModel(ChildModel, Session),
+                [['child field rel to parent',Filter,'parent'],...]),...]
+
+            class ContactModelView(ModelView):
+                datamodel = SQLAModel(Contact, db.session)
+                add_form_query_cascade = [('model', 'manufacture'
+                        SQLAModel(Product, db.session),
+                        [['manufacture_id',FilterEqual,'id']]
+                        )]
+
+    """
+    edit_form_query_cascade = None
+    """
+        FUTURE FEATURE, Don't use it yet
+    """
     add_form_query_rel_fields = None
     """
         Add Customized query for related fields on add form.
@@ -460,14 +469,16 @@ class BaseCRUDView(BaseModelView):
                                              self.description_columns,
                                              self.validators_columns,
                                              self.add_form_extra_fields,
-                                             self.add_form_query_rel_fields)
+                                             self.add_form_query_rel_fields,
+                                             self.add_form_query_cascade)
         if not self.edit_form:
             self.edit_form = conv.create_form(self.label_columns,
                                               self.edit_columns,
                                               self.description_columns,
                                               self.validators_columns,
                                               self.edit_form_extra_fields,
-                                              self.edit_form_query_rel_fields)
+                                              self.edit_form_query_rel_fields,
+                                              self.edit_form_query_cascade)
 
 
     def _init_titles(self):
@@ -653,7 +664,7 @@ class BaseCRUDView(BaseModelView):
 
     def _list(self):
         """
-            list function logic, override to implement diferent logic
+            list function logic, override to implement different logic
             returns list and search widget
         """
         if get_order_args().get(self.__class__.__name__):
@@ -669,6 +680,7 @@ class BaseCRUDView(BaseModelView):
                                         page=page,
                                         page_size=page_size)
         form = self.search_form.refresh()
+        self.update_redirect()
         return self._get_search_widget(form=form, widgets=widgets)
 
 
@@ -683,7 +695,7 @@ class BaseCRUDView(BaseModelView):
 
         widgets = self._get_show_widget(pk)
         item = self.datamodel.get(pk)
-
+        self.update_redirect()
         return self._get_related_views_widgets(item, orders=orders,
                                                pages=pages, page_sizes=page_sizes, widgets=widgets)
 
@@ -693,6 +705,7 @@ class BaseCRUDView(BaseModelView):
             Add function logic, override to implement diferent logic
             returns add widget or None
         """
+        is_valid_form = True
         get_filter_args(self._filters)
         exclude_cols = self._filters.get_relation_cols()
         form = self.add_form.refresh()
@@ -706,6 +719,10 @@ class BaseCRUDView(BaseModelView):
                 self.datamodel.add(item)
                 self.post_add(item)
                 return None
+            else:
+                is_valid_form = False
+        if is_valid_form:
+            self.update_redirect()
         return self._get_add_widget(form=form, exclude_cols=exclude_cols)
 
     def _edit(self, pk):
@@ -713,7 +730,7 @@ class BaseCRUDView(BaseModelView):
             Edit function logic, override to implement diferent logic
             returns Edit widget and related list or None
         """
-
+        is_valid_form = True
         pages = get_page_args()
         page_sizes = get_page_size_args()
         orders = get_order_args()
@@ -725,8 +742,7 @@ class BaseCRUDView(BaseModelView):
         pk = self.datamodel.get_pk_value(item)
 
         if request.method == 'POST':
-            form = self.edit_form(request.form)
-            form = form.refresh(obj=item)
+            form = self.edit_form.refresh(request.form)
             # fill the form with the suppressed cols, generated from exclude_cols
             self._fill_form_exclude_cols(exclude_cols, form)
             # trick to pass unique validation
@@ -737,12 +753,16 @@ class BaseCRUDView(BaseModelView):
                 self.datamodel.edit(item)
                 self.post_update(item)
                 return None
+            else:
+                is_valid_form = False
         else:
-            form = self.edit_form(obj=item)
-            form = form.refresh(obj=item)
+            # Only force form refresh for select cascade events
+            form = self.edit_form.refresh(obj=item)
         widgets = self._get_edit_widget(form=form, exclude_cols=exclude_cols)
         widgets = self._get_related_views_widgets(item, filters={},
                                                   orders=orders, pages=pages, page_sizes=page_sizes, widgets=widgets)
+        if is_valid_form:
+            self.update_redirect()
         return widgets
 
 
@@ -758,6 +778,7 @@ class BaseCRUDView(BaseModelView):
         self.pre_delete(item)
         self.datamodel.delete(item)
         self.post_delete(item)
+        self.update_redirect()
 
 
     """
