@@ -71,6 +71,7 @@ class BS3TextAreaFieldWidget(widgets.TextArea):
             kwargs['placeholder'] = field.label.text
         return super(BS3TextAreaFieldWidget, self).__call__(field, **kwargs)
 
+
 class BS3PasswordFieldWidget(widgets.PasswordInput):
     def __call__(self, field, **kwargs):
         kwargs['class'] = u'form-control'
@@ -80,14 +81,36 @@ class BS3PasswordFieldWidget(widgets.PasswordInput):
 
 
 class Select2Widget(widgets.Select):
+    extra_classes = None
+
+    def __init__(self, extra_classes=None):
+        self.extra_classes = extra_classes
+        return super(Select2Widget, self).__init__()
+
     def __call__(self, field, **kwargs):
-        kwargs['class'] = u'my_select2'
+        if self.extra_classes:
+            kwargs['class'] = u'my_select2 ' + self.extra_classes
+        else:
+            kwargs['class'] = u'my_select2'
         kwargs['style'] = u'width:250px'
         kwargs['data-placeholder'] = u'Select Value'
         if 'name_' in kwargs:
             field.name = kwargs['name_']
         return super(Select2Widget, self).__call__(field, **kwargs)
 
+
+
+class Select2MasterWidget(widgets.Select):
+    def __call__(self, field, **kwargs):
+        kwargs['class'] = u'my_select2 my_change'
+        kwargs['style'] = u'width:250px'
+        kwargs['data-placeholder'] = u'Select Value'
+        if 'name_' in kwargs:
+            field.name = kwargs['name_']
+        return super(Select2Widget, self).__call__(field, **kwargs)
+
+
+
 class Select2ManyWidget(widgets.Select):
     def __call__(self, field, **kwargs):
         kwargs['class'] = u'my_select2'
